#=================================================================
# Author: Garrett York
# Date: 2023/11/29
# Description: Class for Virtuagym API
#=================================================================

from .base_api_wrapper import BaseAPIWrapper

class VirtuagymAPI(BaseAPIWrapper):

    #---------------------------------------------------------------------------
    # Constructor
    #---------------------------------------------------------------------------

    def __init__(self, api_key, club_secret, base_url="https://api.virtuagym.com/api/v1/"):
        super().__init__(base_url)  # Call to parent class constructor
        self.api_key = api_key
        self.club_secret = club_secret
    
    #---------------------------------------------------------------------------
    # Method - Fetch Data
    #---------------------------------------------------------------------------
    
    def _fetch_data(self, endpoint, initial_params):
        self.logger.debug(f'Entering _fetch_paginated_data() for endpoint {endpoint}')
        all_data_list = []
        params = initial_params

        while True:
            response = self.get(endpoint, params=params)
            response = response.json() if response is not None else None
            if response is None:
                self.logger.error(f"Error fetching data from {endpoint}. The response is None.")
                break

            all_data_list.extend(response.get("result", [])) # .extend() adds the elements of the list to the end of the current list

            if response.get("status", {}).get("results_remaining", 0) > 0:
                next_page_str = response["status"]["next_page"]
                if '=' in next_page_str:
                    params['sync_from'] = next_page_str.split('=')[1]
                    self.logger.info(f"Fetching next page: {params['sync_from']} remaining {response['status']['results_remaining']}")
                else:
                    break
            else:
                break
        self.logger.debug("Exiting _fetch_paginated_data()")
        return all_data_list

    #---------------------------------------------------------------------------
    # Method - Get Club Members
    #---------------------------------------------------------------------------

    def get_club_events(self, club_id, timestamp_start=None, timestamp_end=None, member_id=None, schedule_id=None):

        """
        Retrieve all events of a club based on the supplied queries.

        https://github.com/virtuagym/Virtuagym-Public-API/wiki/Club-Events

        :param club_id: ID of the club.
        :param timestamp_start: Start of the time range (milliseconds, optional).
        :param timestamp_end: End of the time range (milliseconds, optional).
        :param member_id: ID of the member (optional).
        :param schedule_id: ID of the schedule (optional).
        :return: List containing the events.
        """

        self.logger.debug(f'Entering get_club_events() for club id {club_id}')

        all_data_list = []
        endpoint = f"club/{club_id}/events/"

        params = {
            "timestamp_start": timestamp_start,
            "timestamp_end": timestamp_end,
            "member_id": member_id,
            "schedule_id": schedule_id
        }
        
        params = self._prepare_params(params, {"api_key": self.api_key, "club_secret": self.club_secret})

        all_data_list = self._fetch_data(endpoint, params)

        self.logger.debug("Exiting get_club_events()")

        return all_data_list
    
    #---------------------------------------------------------------------------
    # Method - Get Club Members
    #---------------------------------------------------------------------------

    def get_event_participants(self, club_id, event_id=None, timestamp_start=None, timestamp_end=None, fill_guestname=False, schedule_id=None):

        """
        Retrieve event participants of a club based on the supplied queries.

        https://github.com/virtuagym/Virtuagym-Public-API/wiki/Club-Event-Participants

        :param club_id: ID of the club.
        :param event_id: ID of the event.
        :param timestamp_start: Start of the time range (milliseconds, optional).
        :param timestamp_end: End of the time range (milliseconds, optional).
        :param fill_guestname: Boolean indicating whether to fill guest names.
        :param schedule_id: ID of the schedule (optional).
        :return: List containing the event participants.
        """

        self.logger.debug("Entering get_event_participants()")

        all_data_list = []
        endpoint = f"club/{club_id}/eventparticipants/"

        params = {
            "timestamp_start": timestamp_start,
            "timestamp_end": timestamp_end,
            "event_id": event_id,
            "schedule_id": schedule_id,
            "fill_guestname": fill_guestname
        }

        params = self._prepare_params(params, {"api_key": self.api_key, "club_secret": self.club_secret})

        all_data_list = self._fetch_data(endpoint, params)

        self.logger.debug("Exiting get_event_participants()")

        return all_data_list
    
    #---------------------------------------------------------------------------
    # Method - Get Club Members
    #---------------------------------------------------------------------------
    
    def get_club_members(self, club_id, club_member_id=None, sync_from=None, from_id=None, with_options=None, any_sub_club=None, rfid_tag=None, external_id=None, email=None, max_results=None):
        
        """
        Retrieve all members of a club or a specific member based on supplied queries.
        
        https://github.com/virtuagym/Virtuagym-Public-API/wiki/Club-Members

        :param club_id: ID of the club.
        :param club_member_id: ID of the specific club member (optional).
        :param sync_from: Timestamp in milliseconds from which to synchronize (optional).
        :param from_id: Starting point for member IDs (optional).
        :param with_options: Additional data to retrieve (optional).
        :param any_sub_club: Whether to include members from any sub-club (optional).
        :param rfid_tag: RFID tag of the member (optional).
        :param external_id: External ID of the member (optional).
        :param email: Email of the member (optional).
        :param max_results: Maximum number of results to retrieve (optional).
        :return: List containing the member(s) information.
        """
        self.logger.debug("Entering get_club_members()")
        
        all_data_list = []

        if club_member_id:
            endpoint = f"club/{club_id}/member/{club_member_id}"
        else:
            endpoint = f"club/{club_id}/member"

        params = {
            "sync_from": sync_from,
            "from_id": from_id,
            "with": with_options,
            "any_sub_club": any_sub_club,
            "rfid_tag": rfid_tag,
            "external_id": external_id,
            "email": email,
            "max_results": max_results
        }

        params = self._prepare_params(params, {"api_key": self.api_key, "club_secret": self.club_secret})

        all_data_list = self._fetch_data(endpoint, params)

        self.logger.debug("Exiting get_club_members()")

        return all_data_list


# =================================================================
# Author: Diego Plata
# Date: 2023/12/19
# Description: Class for Brightpearl API
# =================================================================

from base_api_wrapper import BaseAPIWrapper
from datetime import datetime
from brightpearl_utils.helper_functions import *

class BrightpearlApi(BaseAPIWrapper):

    # -----------------------------------------------------------------
    # Method - Constructor
    # -----------------------------------------------------------------

    def __init__(self, client_id, client_secret, refresh_token, api_iteration_limit=50,
                 auth_url="https://oauth.brightpearl.com", base_url=""):
        super().__init__(base_url)
        self.auth_url = auth_url
        self.client_id = client_id
        self.client_secret = client_secret
        self.refresh_token = refresh_token
        self.api_iteration_limit = api_iteration_limit  # Number of times to call an API endpoint in a single function
        self.time_of_last_refresh = None  # Set in _refresh_access_token()
        self.access_token = None  # Set in _refresh_access_token()
        self._refresh_access_token()

    # -----------------------------------------------------------------
    # Method - Refresh Access Token
    # -----------------------------------------------------------------

    def _refresh_access_token(self):

        """
        Refreshes the OAuth access token using the refresh token.

        :return: True if the access token was successfully refreshed, False otherwise.
        """

        self.logger.info("Entering _refresh_access_token()")
        self.time_of_last_refresh = datetime.now()

        path = "/token/driveline"

        data = {
            'refresh_token': self.refresh_token,
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': 'refresh_token'
        }
        headers = {}

        response = self.post(path=path, headers=headers, data=data, is_auth=True)
        response = response.json() if response is not None else None

        # valid if response is successful, if it was return true, if not return false
        if response is None:
            self.logger.error("Failed to refresh access token - response is None")
            return False
        else:
            # if access token exists, return true
            if "access_token" in response:
                self.access_token = response["access_token"]
                self.base_url = f"https://{response['api_domain']}"
                self.logger.info("Access token refreshed and base url set to api domain")
                return True
            else:
                self.logger.error("Failed to refresh access token - access token does not exist in response")
                return False

    # -----------------------------------------------------------------
    # Method - Check Last Time Access Token Was Refreshed
    # -----------------------------------------------------------------

    def check_if_access_token_needs_refreshed(self):
        """
        Checks if the access token needs to be refreshed.
        return: True if the access token needs to be refreshed, False otherwise.
        """
        self.logger.info("Entering check_last_refresh()")

        time_since_last_refresh = datetime.now() - self.time_of_last_refresh

        if time_since_last_refresh.seconds > 3600:
            self.logger.info("Exiting check_last_refresh() - Token needs to be refreshed")
            return True
        else:
            self.logger.info("Exiting check_last_refresh() - Token does not need to be refreshed")
            return False

    # -----------------------------------------------------------------
    # Method - Get Headers
    # -----------------------------------------------------------------

    def _get_headers(self):
        self.logger.info("Entering _get_headers()")
        if self.check_if_access_token_needs_refreshed():
            if not self._refresh_access_token():
                return None

        headers = {
            'Authorization': f"Bearer {self.access_token}",
            'Content-Type': 'application/json',
            'brightpearl-app-ref': self.client_id,
            'brightpearl-dev-ref': 'driveline1058'
        }

        self.logger.info(f'Headers have been set!')
        self.logger.info("Exiting _get_headers()")
        return headers

    def get_all_sales_orders_from_date(self, since_date, page_size):
        # TODO: add docstring documentation
        self.logger.info("Entering get_all_sales_orders()")
        endpoint = "/public-api/driveline/order-service/sales-order-search"

        result_index = 1
        today = pd.to_datetime('today').date()
        params = {
                    "createdOn": f"{since_date}T00:00/{today}T00:00",
                    "pageSize": page_size,
                    "firstResult": result_index
        }

        sales_orders = self._fetch_sales_orders_by_date(self.base_url+endpoint, params)

        return sales_orders

    def get_all_sales_orders_by_ids(self, sales_orders):
        # TODO: add docstring documentation
        self.logger.info("Entering get_all_sales_orders_by_id()")
        order_service_endpoint = "/public-api/driveline/order-service"
        sales_order_endpoint = "/public-api/driveline/order-service/sales-order"
        params = {

        }
        headers = self._get_headers()
        orders_ids = construct_sales_orders_url(sales_orders['salesOrderId'].values, self.base_url)
        uris_response = self.options(f"{self.base_url}{sales_order_endpoint}/{','.join(orders_ids)}", headers=headers)
        uris_response = uris_response.json() if uris_response is not None else None
        urls = uris_response["response"]["getUris"]
        sales_orders = self._fetch_sales_orders_by_ids(f"{self.base_url}{order_service_endpoint}", params, urls)

        return sales_orders

    def _fetch_sales_orders_by_date(self, endpoint, initial_params):
        # TODO: add docstring documentation
        self.logger.info(f'Entering _fetch_data() for endpoint {endpoint}')
        all_data_list = []
        params = initial_params
        headers = self._get_headers()
        more_pages_available = True
        while more_pages_available:
            response = self.get(endpoint, params=params, headers=headers)
            response = response.json() if response is not None else None
            more_pages_available = response["response"]["metaData"]["morePagesAvailable"]
            data_count = response["response"]["metaData"]["resultsReturned"]
            first_index_returned = response["response"]["metaData"]["firstResult"]
            last_index_returned = response["response"]["metaData"]["lastResult"]
            self.logger.info(f"Retrieved {data_count} results from index {first_index_returned} to index {last_index_returned}")
            all_data_list.append(sales_orders_list_to_dataframe(response["response"]["results"], response["response"]["metaData"]["columns"]))
            result_index = response["response"]["metaData"]["lastResult"] + 1
            params['firstResult'] = result_index

        self.logger.info(f"Exiting _fetch_sales_orders_by_date()")
        return pd.concat(all_data_list)

    def _fetch_sales_orders_by_ids(self, endpoint, initial_params, urls):
        # TODO: add docstring documentation
        self.logger.info(f'Entering _fetch_data() for endpoint {endpoint}')
        params = initial_params
        headers = self._get_headers()
        response = [self.get(endpoint+url, params=params, headers=headers) for url in urls]
        response = [response.json() if response is not None else None for response in response]
        individual_sales_orders_list = list()
        for sale_order_dictionary in response:
            individual_sales_orders_list.append(individual_sales_orders_to_dataframe(sale_order_dictionary["response"]))
        self.logger.info(f"Exiting _fetch_sales_orders_by_ids()")
        return pd.concat(individual_sales_orders_list)


import pandas as pd


def sales_orders_list_to_dataframe(results, columns):
    header_columns = [column["name"] for column in columns]
    return pd.DataFrame(results, columns=header_columns)

def construct_sales_orders_url(ids, base_url):
    string_ids = [str(id) for id in ids]
    return string_ids

def individual_sales_orders_to_dataframe(orders):
    return pd.json_normalize(orders, sep='_')


def append_order_and_individual_sales_entry(orders):
    row_frames = list()
    for _, row in orders.iterrows():
        row_frame = pd.json_normalize(row["rows"])
        # can join later
        row_frame["salesOrderId"] = row["id"]
        row_frame = row_frame.add_prefix("individual_sale_entry_")
        row_frames.append(row_frame)
    row_frames = pd.concat(row_frames)
    megaframe = orders.merge(row_frames, how='right',left_on=["id"], right_on=["individual_sale_entry_salesOrderId"], suffixes=("","sale_entry"))
    megaframe = megaframe.drop("rows",axis=1)
    return megaframe


import unittest
from unittest.mock import patch, MagicMock
from drivelinepy.virtuagym_api import VirtuagymAPI
import os
from dotenv import load_dotenv
import logging

# Load environment variables
load_dotenv()

# Configure logging
logging.basicConfig(level=logging.INFO)

class TestVirtuagymAPI(unittest.TestCase):

    def setUp(self):
        self.api_key = os.getenv("DRIVELINEPY_VIRTUAGYM_API_KEY")
        self.club_secret = os.getenv("DRIVELINEPY_VIRTUAGYM_CLUB_SECRET")
        self.club_id = os.getenv("DRIVELINEPY_VIRTUAGYM_WASHINGTON_CLUB_ID")
        
        if not all([self.api_key, self.club_secret, self.club_id]):
            raise ValueError("Virtuagym API credentials or club ID not set in environment variables")
        
        self.virtuagym_api = VirtuagymAPI(api_key=self.api_key, club_secret=self.club_secret)
        self.test_member_id = "27119834"  # garrett york id
        self.expected_email = "garrett@drivelinebaseball.com"

    @patch('drivelinepy.virtuagym_api.BaseAPIWrapper.get')
    def test_get_single_club_member_email(self, mock_get):
        logging.info("Virtuagym API - Testing get_club_members for single member email...")

        mock_response = MagicMock()
        mock_response.json.return_value = {
            "result": [
                {
                    "member_id": 27119834,
                    "email": self.expected_email,
                    # Other fields omitted for brevity
                }
            ],
            "status": {
                "results_remaining": 0
            }
        }
        mock_get.return_value = mock_response

        # Call the method to get a single club member
        member = self.virtuagym_api.get_club_members(self.club_id, club_member_id=self.test_member_id)

        # Assertions
        self.assertIsInstance(member, list, "Result should be a list")
        self.assertEqual(len(member), 1, "Result should contain exactly one member")
        self.assertEqual(member[0]['email'], self.expected_email, "Email should match the expected value")

        # Verify that the API was called with the correct parameters
        mock_get.assert_called_once()
        args, kwargs = mock_get.call_args
        self.assertIn(f"club/{self.club_id}/member/{self.test_member_id}", args[0], 
                      "API call should include correct club_id and member_id")

if __name__ == '__main__':
    unittest.main()


import unittest
from unittest.mock import patch, MagicMock
from drivelinepy.zoho_crm_api import ZohoCrmAPI
from dotenv import load_dotenv
import logging
import os

# Load environment variables
load_dotenv()

# Configure logging
logging.basicConfig(level=logging.INFO)

class TestZohoCrmAPI(unittest.TestCase):

    def setUp(self):
        print("Zoho CRM API - Setting up class for testing...")
        self.client_id = os.getenv("DRIVELINEPY_ZOHO_CLIENT_ID")
        self.client_secret = os.getenv("DRIVELINEPY_ZOHO_CLIENT_SECRET")
        self.refresh_token = os.getenv("DRIVELINEPY_ZOHO_REFRESH_TOKEN")
        self.organization_id = os.getenv("DRIVELINEPY_ZOHO_ORGANIZATION_ID")

        if not all([self.client_id, self.client_secret, self.refresh_token, self.organization_id]):
            raise ValueError("Zoho CRM API credentials not set in environment variables")

        self.zoho = ZohoCrmAPI(
            client_id=self.client_id,
            client_secret=self.client_secret,
            refresh_token=self.refresh_token,
            organization_id=self.organization_id
        )

    @patch('drivelinepy.zoho_crm_api.ZohoCrmAPI._fetch_data')
    def test_get_specific_contact_email(self, mock_fetch_data):
        print("Zoho CRM API - Testing get_customer_records for email confirmation...")
        
        expected_email = "garrett@drivelinebaseball.com"
        
        # Prepare mock data
        mock_contact = [{
            "Email": expected_email,
        }]
        mock_fetch_data.return_value = mock_contact

        # Call the method
        contact = self.zoho.get_customer_records(module="Contacts", email=expected_email)

        # Assertions for the returned data
        self.assertIsInstance(contact, list)
        self.assertEqual(len(contact), 1)
        self.assertEqual(contact[0]["Email"], expected_email)

        # Verify that the method was called correctly
        mock_fetch_data.assert_called_once()
        
        # Inspect the call arguments
        args, kwargs = mock_fetch_data.call_args
        
        # Assert the correct endpoint and search criteria
        self.assertEqual(args[0], "Contacts/search")
        self.assertIn("initial_params", kwargs)
        self.assertIn("criteria", kwargs["initial_params"])
        self.assertEqual(kwargs["initial_params"]["criteria"], f"(Email:equals:{expected_email})")

if __name__ == '__main__':
    unittest.main()


# ================================================================================
# Author: Garrett York
# Date: 2024/01/31
# Description: Class for TRAQ API
# ================================================================================

from .base_api_wrapper import BaseAPIWrapper


class TRAQAPI(BaseAPIWrapper):

    # 0 = Inactive, 1 = On-Site, 2 = Remote, 3 = Active, 4 = Archived
    VALID_STATUS_CODES = [0, 1, 2, 3, 4]

    # 0 = No Sub-Status, 217 = No Facility, 31 = AZ, 215 = WA
    VALID_SUBSTATUS_CODES = [0, 217, 31, 215]

    # ---------------------------------------------------------------------------
    # Constructor
    # ---------------------------------------------------------------------------

    def __init__(self, client_id, client_secret, auth_url="https://traq.drivelinebaseball.com",
                 base_url="https://traq.drivelinebaseball.com/"):

        super().__init__(base_url)
        self.auth_url = auth_url
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token = None
        self.authenticate()

    # ---------------------------------------------------------------------------
    # Method - Authenticate
    # ---------------------------------------------------------------------------

    def authenticate(self):
        """
        Authenticates with the TRAQ API and sets the access token.
        """
        self.logger.info("Entering TRAQ API authenticate()")
        path = "oauth/token"
        payload = {
            'grant_type': 'client_credentials',
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'scope': '*'
        }
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        response = self.post(path=path, data=payload, headers=headers, is_auth=True)
        response = response.json() if response is not None else None

        if response:
            self.access_token = response.get('access_token')
            self.logger.info("Authentication successful")
        else:
            self.logger.error("Authentication failed")

        self.logger.info("Exiting authenticate()")

    #---------------------------------------------------------------------------
    # Method - Get Users
    # ---------------------------------------------------------------------------

    def get_users(self, traq_id=None, email=None, status_code=None, substatus_code=None, facility_id=None):
        """
        Retrieves user information from the TRAQ API. Prioritizes TRAQ ID over email and status code.

        :param traq_id: TRAQ ID to filter users (optional).
        :param email: Email address to filter users (optional).
        :param status_code: Status code (int) to filter users (optional).
        :param facility_id: Facility ID to filter users (optional).
        :param substatus_code: Substatus code (int) to filter users (optional). 

        :return: User information or list of users.
        """
        self.logger.debug("Entering get_users()")

        endpoint = "api/v1.1/users"
        headers = {'Authorization': f'Bearer {self.access_token}'}
        params = {}

        # Check for facility filter
        if facility_id:
            if self.validate_facility_id(facility_id):
                params['facility_id'] = facility_id
            else:
                return None
        else:
            # if email and traq_id are not provided, warn the user that a facility ID filter is recommended
            if not traq_id and not email:
                self.logger.warning("Facility ID filter was not provided. It is highly recommended to provide a facility ID to avoid large data sets if not pinged by email or TRAQ ID.")

        # Check for unique identifyier filter
        if traq_id:
            if self.validate_traq_id(traq_id):
                params = {'id': traq_id}
            else:
                return None
        elif email:
            if self.validate_email(email):
                params = {'email': email}
            else:
                return None
        else:
            self.logger.warning("Neither TRAQ ID or Email provided")

        # Add status_code to params if provided
        if status_code:
            if self.validate_traq_user_status_code(status_code):
                params = {"active": status_code}
            else:
                return None
        else:
            self.logger.debug("Status code filter was not provided")
        
        # Add substatus_code to params if provided -- traq naming convention is a bit wonky here because "status" is actually the substatus_code
        if substatus_code:
            if self.validate_traq_user_substatus_code(substatus_code):
                params['status'] = substatus_code
            else:
                return None
        else:
            self.logger.debug("Substatus code filter was not provided")

        if not params:
            self.logger.debug("No valid filters provided.")
            return None

        response = self.get(endpoint, params=params, headers=headers)
        response = response.json() if response is not None else None

        self.logger.debug("Exiting get_users()")
        return response.get('data') if response else None

    #---------------------------------------------------------------------------
    # Method - Get Workouts
    #---------------------------------------------------------------------------

    def get_workouts(self, program):
        """
        Retrieves workouts information from the TRAQ API.

        :param program: Program ID to filter workouts (optional).
        :return: Workouts information or list of workouts.
        """
        self.logger.debug("Entering get_workouts()")

        endpoint = "api/v1.1/list-workouts"
        headers = {'Authorization': f'Bearer {self.access_token}'}
        if self.validate_traq_program(program):
            params = {"program": program}
        else:
            return None  # Invalid program - logging done in validate_traq_program

        response = self.get(endpoint, params=params, headers=headers)
        response = response.json() if response is not None else None

        self.logger.debug("Exiting get_workouts()")
        return response.get('data') if response else None

    #---------------------------------------------------------------------------
    # Method - Get Workouts by Athlete
    #---------------------------------------------------------------------------

    def get_athlete_workouts(self, traq_id, start_date, end_date):
        """
        Retrieves workouts information from the TRAQ API.

        :param traq_id: TRAQ ID to filter users.
        :param start_date: Starting date range to filter workouts.
        :param end_date: Ending date range to filter workouts.
        :return: Workouts filtered by athlete and date information or list of workouts.
        """
        self.logger.debug("Entering get_athlete_workouts()")

        endpoint = "api/v1.1/athlete-workout"
        headers = {'Authorization': f'Bearer {self.access_token}'}
        params = {}
        if self.validate_traq_id(traq_id) and self.validate_date_yyyy_mm_dd(
                start_date) and self.validate_date_yyyy_mm_dd(end_date):
            params = {"start_date": start_date, "end_date": end_date, "user_id": traq_id}
        else:
            return None  # Invalid params - logging done in validation methods
        response = self.get(endpoint, params=params, headers=headers)
        response = response.json() if response is not None else None
        self.logger.debug("Exiting get_athlete_workouts()")

        return response.get('data') if response else None

    #---------------------------------------------------------------------------
    # Validation Methods
    #---------------------------------------------------------------------------

    def validate_facility_id(self, facility_id):
        """
        Validates that the facility id is an integer within the range of 1 to 999999.
        """
        if isinstance(facility_id, int):
            if 1 <= facility_id <= 999999:
                self.logger.info(f"Facility ID is valid: {facility_id}.")
                return True
            else:
                self.logger.error(f"Invalid Facility ID: {facility_id}. It must be a 1-6 digit integer within the range 1 to 999999.")
                return False
        else:
            self.logger.error(f"Invalid Facility ID: {facility_id}. It must be an integer.")
            return False

    def validate_traq_id(self, traq_id):
        """
        Validates that the TRAQ ID is an integer within the range of 1 to 999999.
        :param traq_id: The TRAQ ID to be validated.
        :return: Boolean indicating whether the ID is valid.
        """
        if isinstance(traq_id, int):
            if 1 <= traq_id <= 999999:
                self.logger.info(f"TRAQ ID is valid: {traq_id}.")
                return True
            else:
                self.logger.error(f"Invalid TRAQ ID: {traq_id}. It must be a 1-6 digit integer within the range 1 to 999999.")
                return False
        else:
            self.logger.error(f"Invalid TRAQ ID: {traq_id}. It must be an integer.")
            return False

    def validate_traq_program(self, program):
        """
        Validates that the TRAQ program is an int representing one of these possible options: -1, 0, 1, 2, 3, 4.

        :param program: The TRAQ program number to be validated.
        :return: Boolean indicating whether the program is valid.
        """
        if isinstance(program, int) and -1 <= program <= 4:
            self.logger.info(f"TRAQ program is valid: {program}. Exiting validate_traq_program()")
            return True
        else:
            self.logger.error(f"Invalid TRAQ program name int: {program}. Possible options: -1, 0, 1, 2, 3, 4.")
            return False

    def validate_traq_user_status_code(self, status_code):
        """
        :param status_code: The TRAQ status_code number to be validated.
        :return: Boolean indicating whether the program is valid.
        """
        if isinstance(status_code, int) and status_code in self.VALID_STATUS_CODES:
            self.logger.info(f"TRAQ status_code int is valid: {status_code}. Exiting validate_traq_user_status_code()")
            return True
        else:
            self.logger.error(f"Invalid TRAQ status_code int: {status_code}. Possible options: " + ", ".join(map(str, self.VALID_STATUS_CODES)) + ".")
            return False
        

    def validate_traq_user_substatus_code(self, substatus_code):
        """
        :param substatus_code: The TRAQ substatus_code number to be validated.
        :return: Boolean indicating whether the program is valid.
        """
        if isinstance(substatus_code, int) and substatus_code in self.VALID_SUBSTATUS_CODES:
            self.logger.info(f"TRAQ substatus_code int is valid: {substatus_code}. Exiting validate_traq_user_substatus_code()")
            return True
        else:
            self.logger.error(f"Invalid TRAQ substatus_code int: {substatus_code}. Possible options: " + ", ".join(map(str, self.VALID_SUBSTATUS_CODES)) + ".")
            return False



#================================================================================
# Author: Garrett York
# Date: 2024/02/01
# Description: Class for Slack API
#================================================================================

from .base_api_wrapper import BaseAPIWrapper
import os
import mimetypes

class SlackAPI(BaseAPIWrapper):

    #---------------------------------------------------------------------------
    # Constructor
    #---------------------------------------------------------------------------

    def __init__(self, token, base_url="https://slack.com/api/"):
        super().__init__(base_url)
        self.token = token

    #---------------------------------------------------------------------------
    # Method - Post Message
    #---------------------------------------------------------------------------

    def post_message(self, channel, text, thread_timestamp=None):
        """
        Posts a message to a specified channel on Slack.

        :param channel: The channel ID where the message will be posted.
        :param text: The text of the message to post.
        :param thread_timestamp: The timestamp (string or float) of the parent message to post in a thread.
                                This should be a UNIX timestamp to 6 decimal places, typically obtained from
                                the response of a successful post.
        :return: The response from the Slack API as a JSON object.
        """
        self.logger.info("Entering post_message()")

        if not text:
            error_msg = "Text cannot be None or an empty string."
            self.logger.error(error_msg)
            raise ValueError(error_msg)
        
        endpoint = "chat.postMessage"
        payload = {
            'channel': channel,
            'text': text
        }
        if thread_timestamp:
            payload['thread_ts'] = thread_timestamp

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': f'Bearer {self.token}'
        }

        response = self.post(endpoint, data=payload, headers=headers)

        if response is None:
            error_msg = "Failed to post message: No response received from Slack API. Please check the network connection and API endpoint."
            self.logger.error(error_msg)
            raise Exception(error_msg)
        
        try:
            response_json = response.json()
        except ValueError as e:
            error_msg = f"Failed to parse JSON response: {e}. Response content: {response.text}"
            self.logger.error(error_msg)
            raise Exception(error_msg)

        if response.status_code != 200 or not response_json.get('ok', False):
            error_msg = (f"Failed to post message to channel {channel}. "
                        f"Status Code: {response.status_code}, Response: {response.text}")
            self.logger.error(error_msg)
            raise Exception(error_msg)

        self.logger.info(f"Message posted successfully to channel {channel}.")
        self.logger.info("Exiting post_message()")
        return response_json
    
    #---------------------------------------------------------------------------
    # Method - Post File + Message (optional)
    #---------------------------------------------------------------------------

    def upload_file(self, channel, file_absolute_path, text=None, thread_timestamp=None):
        """
        Uploads a file to a specified channel on Slack. Optionally posts the
        file in a thread.

        This method attempts to upload a file to the specified Slack channel. It first checks if the file exists at the given path and then determines its MIME type for proper uploading. The file is then uploaded with an optional initial comment.

        :param channel: str
            The channel ID where the file will be uploaded.
        :param file_absolute_path: str
            The absolute path to the file to upload.
        :param text: str, optional
            An initial comment to add when uploading the file. Defaults to None. The API documentation refers to this parameter is initial_comment
            but it is referred to as text in this method to match the parameter name in the post_message method.
        :param thread_timestamp: str, optional
            The timestamp of the parent message to post in a thread. Defaults to None.

        :return: dict
            A dictionary response from the Slack API indicating the success or failure of the file upload.
        """
        self.logger.info("Entering upload_file()")

        if not os.path.exists(file_absolute_path):
            error_msg = f"File not found: {file_absolute_path}"
            self.logger.error(error_msg)
            raise FileNotFoundError(error_msg)

        endpoint = "files.upload"

        mime_type, _ = mimetypes.guess_type(file_absolute_path)
        mime_type = mime_type or 'application/octet-stream'

        try:
            with open(file_absolute_path, 'rb') as file_content:
                files = [('file', (os.path.basename(file_absolute_path), file_content, mime_type))]
                payload = {'channels': channel}
                if text:
                    payload['initial_comment'] = text
                if thread_timestamp:
                    payload['thread_ts'] = thread_timestamp
                headers = {'Authorization': f'Bearer {self.token}'}
                response = self.post(endpoint, headers=headers, data=payload, files=files)
                
                if response is None:
                    error_msg = "Failed to upload file: No response received from Slack API. Please check the network connection and API endpoint."
                    self.logger.error(error_msg)
                    raise Exception(error_msg)
                
                try:
                    response_json = response.json()
                except ValueError as e:
                    error_msg = f"Failed to parse JSON response: {e}. Response content: {response.text}"
                    self.logger.error(error_msg)
                    raise Exception(error_msg)
                
                if response.status_code != 200 or not response_json.get('ok', False):
                    error_msg = (f"Failed to upload file to channel {channel}. "
                                f"Status Code: {response.status_code}, Response: {response.text}")
                    self.logger.error(error_msg)
                    raise Exception(error_msg)
                
        except Exception as e:
            error_msg = f"Error uploading file: {e}"
            self.logger.error(error_msg)
            raise Exception(error_msg)

        self.logger.info("Exiting upload_file()")
        return response_json


import unittest
from unittest.mock import patch
from drivelinepy.zoho_billing_api import ZohoBillingAPI
from dotenv import load_dotenv
import os
import logging

# Load environment variables
load_dotenv()

# Configure logging
logging.basicConfig(level=logging.INFO)

class TestZohoBillingAPI(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        """Set up the test environment once for all test methods."""
        print("Zoho Billing API - Setting up class for testing...")
        cls.client_id = os.getenv("DRIVELINEPY_ZOHO_CLIENT_ID")
        cls.client_secret = os.getenv("DRIVELINEPY_ZOHO_CLIENT_SECRET")
        cls.refresh_token = os.getenv("DRIVELINEPY_ZOHO_REFRESH_TOKEN")
        cls.organization_id = os.getenv("DRIVELINEPY_ZOHO_ORGANIZATION_ID")

        if not all([cls.client_id, cls.client_secret, cls.refresh_token, cls.organization_id]):
            raise ValueError("Zoho Billing API credentials not set in environment variables")

        cls.zoho_billing_api = ZohoBillingAPI(
            client_id=cls.client_id,
            client_secret=cls.client_secret,
            refresh_token=cls.refresh_token,
            organization_id=cls.organization_id
        )

    @patch('drivelinepy.zoho_billing_api.ZohoBillingAPI._fetch_data')
    def test_get_customer_by_email(self, mock_fetch_data):
        """Test getting a customer by email address."""
        print("Zoho Billing API - Testing get_customers for email confirmation...")
        
        test_email = "garrett@drivelinebaseball.com"
        expected_customer_data = {"email": test_email}
        
        # Mock the API response
        mock_fetch_data.return_value = [expected_customer_data]

        # Call the method being tested
        result = self.zoho_billing_api.get_customers(email=test_email)

        # Assertions
        self.assertEqual(len(result), 1, "Result should contain exactly one customer")
        self.assertEqual(result[0]["email"], test_email, "Returned email should match the test email")

        # Verify that _fetch_data was called with the correct parameters
        mock_fetch_data.assert_called_once_with(
            'customers',
            {'email': test_email},
            'customers',
            page=1
        )

if __name__ == '__main__':
    unittest.main()


from .base_api_wrapper import BaseAPIWrapper
from .zoho_billing_api import ZohoBillingAPI
from .zoho_crm_api import ZohoCrmAPI
from .virtuagym_api import VirtuagymAPI
from .traq_api import TRAQAPI
from .slack_api import SlackAPI



import unittest
from unittest.mock import patch, MagicMock
from drivelinepy.traq_api import TRAQAPI
import os
from dotenv import load_dotenv
import logging
import time

# Load environment variables
load_dotenv()

# Configure logging
logging.basicConfig(level=logging.INFO)

class TestTRAQAPI(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("TRAQ API - Setting up class for testing...")
        cls.client_id = os.getenv("DRIVELINEPY_TRAQ_CLIENT_ID")
        cls.client_secret = os.getenv("DRIVELINEPY_TRAQ_CLIENT_SECRET")
        if not cls.client_id or not cls.client_secret:
            raise ValueError("TRAQ API credentials not set in environment variables")
        cls.traq_api = TRAQAPI(cls.client_id, cls.client_secret)
        cls.VALID_TRAQ_ID = 6183
        cls.VALID_EMAIL = "garrettyork03@gmail.com"
        cls.mock_response = {"data": [
            {'id': 6183, 'email': 'garrettyork03@gmail.com'}
        ]}

    #-----------------------------------------------------------------
    # Test - Get users by TRAQ ID
    #----------------------------------------------------------------

    @patch('drivelinepy.traq_api.BaseAPIWrapper.get')
    def test_get_users_by_traq_id(self, mock_get):
        print("TRAQ API - Testing get_users by TRAQ ID...")
        mock_get.return_value.json.return_value = self.mock_response
        response = self.traq_api.get_users(traq_id=self.VALID_TRAQ_ID)
        
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0]['id'], self.VALID_TRAQ_ID)
        self.assertEqual(response[0]['email'], 'garrettyork03@gmail.com')

    #-----------------------------------------------------------------
    # Test - Get users by email
    #-----------------------------------------------------------------

    @patch('drivelinepy.traq_api.BaseAPIWrapper.get')
    def test_get_users_by_email(self, mock_get):
        print("TRAQ API - Testing get_users by email...")
        mock_get.return_value.json.return_value = self.mock_response
        response = self.traq_api.get_users(email=self.VALID_EMAIL)
        
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 1)
        self.assertEqual(response[0]['id'], self.VALID_TRAQ_ID)
        self.assertEqual(response[0]['email'], 'garrettyork03@gmail.com')

if __name__ == '__main__':
    unittest.main()


from setuptools import setup, find_packages
import os

# print working directory
print("Current working directory: ", os.getcwd())
print("Contents of current working directory: ", os.listdir())

# Read the contents of your README file
with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='drivelinepy',
    version='1.9.2',
    author='Garrett York',
    author_email='garrett@drivelinebaseball.com',
    description='A Python package for Driveline Baseball API interactions',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/drivelineresearch/drivelinepy',
    packages=find_packages(),
    install_requires=[
        'certifi>=2024.2.2,<2025.0.0',
        'charset-normalizer>=3.3.2,<4.0.0',
        'idna>=3.6,<4.0.0',
        'numpy>=1.24.4,<1.26.0',
        'pandas>=1.4.3,<2.0.0',
        'python-dateutil>=2.8.2,<3.0.0',
        'python-dotenv>=1.0.1,<2.0.0',
        'pytz>=2024.1,<2025.0.0',
        'requests>=2.31.0,<3.0.0',
        'six>=1.16.0,<2.0.0',
        'tzdata>=2024.1,<2025.0.0',
        'urllib3>=2.2.0,<3.0.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.8',
)